@@ -85,6 +85,9 @@ ControlWidget::ControlWidget(const QString &ruleFile, Control *parent)
     connect(ui->pbStart, SIGNAL(clicked()), this, SLOT(simAppStart()));
     connect(ui->pbAbort, SIGNAL(clicked()), this, SLOT(simAppAbort()));
     connect(ui->pbReset, SIGNAL(clicked()), this, SLOT(modemSilentReset()));
+    connect(ui->pbSendGNSSData, SIGNAL(clicked()), this, SLOT(sendGNSSData()));
+    connect(ui->pbGNSSDefault, SIGNAL(clicked()), this, SLOT(setDefaultGNSSData()));
+    connect(ui->pbSendNotif, SIGNAL(clicked()), this, SLOT(sendCSSN()));
 
     QStringList headers;
     headers << "Sender" << "Priority" << "Notification Status";
@@ -92,6 +95,7 @@ ControlWidget::ControlWidget(const QString &ruleFile, Control *parent)
     ui->twMessageList->verticalHeader()->hide();
 
     handleNewApp();
+    handleCSSNNotif();
 
     show();
 }
@@ -119,7 +123,7 @@ Control::Control(const QString& ruleFile, SimRules *sr, QObject *parent)
         << SIGNAL(command(QString))
         << SIGNAL(variableChanged(QString,QString))
         << SIGNAL(switchTo(QString))
-        << SIGNAL(startIncomingCall(QString));
+        << SIGNAL(startIncomingCall(QString, QString, QString));
 
     foreach (QByteArray sig, proxySignals)
         connect(widget, sig, this, sig);
@@ -140,13 +144,39 @@ void Control::warning( const QString &title, const QString &message )
     QMessageBox::warning(widget, title, message, "OK");
 }
 
+void ControlWidget::handleCSSNNotif()
+{
+    ui->cbCSSU->insertItem(0, "");
+    ui->cbCSSU->insertItem(1, "0 - forwarded", 0);
+    ui->cbCSSU->insertItem(3, "2 - on hold", 2);
+    ui->cbCSSU->insertItem(4, "3 - retrieved", 3);
+    ui->cbCSSU->insertItem(5, "4 - multiparty", 4);
+
+    ui->cbCSSI->insertItem(0, "");
+    ui->cbCSSI->insertItem(3, "2 - forwarded", 2);
+    ui->cbCSSI->insertItem(6, "5 - outgoing barred", 5);
+    ui->cbCSSI->insertItem(7, "6 - incomming barred", 6);
+}
+
+void ControlWidget::sendCSSN()
+{
+    QVariant v = ui->cbCSSU->itemData(ui->cbCSSU->currentIndex());
+
+    if (v.canConvert<int>())
+        emit unsolicitedCommand("+CSSU: "+QString::number(v.toInt()));
+
+    v = ui->cbCSSI->itemData(ui->cbCSSI->currentIndex());
+
+    if (v.canConvert<int>())
+        emit unsolicitedCommand("+CSSI: "+QString::number(v.toInt()));
+}
+
 void ControlWidget::sendSQ()
 {
     emit variableChanged("SQ",QString::number(ui->hsSignalQuality->value())+",99");
     emit unsolicitedCommand("+CSQ: "+QString::number(ui->hsSignalQuality->value())+",99");
 }
 
-
 void ControlWidget::sendBC()
 {
     bool charging = ui->hsBatteryCharging->checkState() == Qt::Checked;
@@ -222,6 +252,33 @@ void ControlWidget::sendSMSMessage()
     p->constructSMSMessage(ui->leSMSClass->text().toInt(), ui->leMessageSender->text(), ui->leSMSServiceCenter->text(), ui->teSMSText->toPlainText());
 }
 
+void ControlWidget::setDefaultGNSSData()
+{
+    ui->teGNSStext->clear();
+
+    ui->teGNSStext->append("<?xml version=\"1.0\" ?>");
+    ui->teGNSStext->append("<pos>");
+    ui->teGNSStext->append("  <pos_err>");
+    ui->teGNSStext->append("    <err_reason literal=\"not_enough_gps_satellites\" />");
+    ui->teGNSStext->append("  </pos_err>");
+    ui->teGNSStext->append("</pos>");
+}
+
+void ControlWidget::sendGNSSData()
+{
+    QStringList xml = ui->teGNSStext->toPlainText().split("\n");
+
+    foreach ( QString line, xml )
+    {
+        if (!line.isEmpty()) {
+            QString cposr =  "+CPOSR: " + line;
+
+            emit unsolicitedCommand(cposr);
+            handleFromData( cposr );
+        }
+    }
+}
+
 void ControlWidget::sendMGD()
 {
     emit command("AT+CMGD=1");
@@ -281,7 +338,20 @@ void ControlWidget::sendSMSDatagram()
 
 void ControlWidget::sendCall()
 {
-    emit startIncomingCall( ui->leCaller->text() );
+    QString number;
+    QString calledNumber;
+    QString name;
+
+    if (ui->cbCaller->isChecked())
+        number = ui->leCaller->text();
+
+    if (ui->cbCalledLine->isChecked())
+        calledNumber = ui->leCalledLine->text();
+
+    if (ui->cbCallerName->isChecked())
+        name = ui->leCallerName->text();
+
+    emit startIncomingCall( number, calledNumber, name );
 }
 
 void ControlWidget::handleFromData( const QString& cmd )
@@ -448,8 +518,7 @@ void ControlWidget::simInsertRemove()
 
 void ControlWidget::handleNewApp()
 {
-    ui->lblApplicationName->setText( "Current application: " +
-            p->getSimAppName() );
+    ui->cbSimApps->insertItems( 0, p->getSimAppsNameList() );
 }
 
 void Control::handleNewApp()
@@ -459,7 +528,8 @@ void Control::handleNewApp()
 
 void ControlWidget::simAppStart()
 {
-    p->simAppStart();
+    if ( ui->cbSimApps->currentIndex() >= 0 )
+        p->simAppStart( ui->cbSimApps->currentIndex() );
 }
 
 void ControlWidget::simAppAbort()
@@ -492,6 +562,9 @@ Script::Script(QObject *obj, Ui_ControlBase *ui) : QDBusAbstractAdaptor(obj)
 
     QScriptValue qsTab6 = engine.newQObject(ui->tab_6);
     engine.globalObject().setProperty("tabSIM", qsTab6);
+
+    QScriptValue qsTab8 = engine.newQObject(ui->tab_8);
+    engine.globalObject().setProperty("tabPosition", qsTab8);
 }
 
 void Script::SetPath(const QString &path, const QDBusMessage &msg)
